//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_TreeNode.java,v 1.9 2000/12/05 03:05:04 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_TreeNode.java,v $
//  Revision 1.9  2000/12/05 03:05:04  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.8  2000/06/22 17:22:49  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.7  2000/04/06 19:13:25  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.6  2000/03/03 18:38:30  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.5  2000/02/10 00:37:34  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.3  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/13 20:01:29  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:50  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:46  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/08/01 20:54:38  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.1  1999/08/01 17:00:12  rkamath
//  Added new files for overload resolution
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.lang.Object;
import java.io.*;

/** This represents the <B>TreeNode</B> object
 * The Tree was written with the express purpose of implementing Overloading
 * resolution as outlined in "Crafting a Compiler with C" authored by
 * Charles N. Fischer and Richard J. LeBlanc Jr. pp 420-422 which is
 * essentially Baker's Bottom up resolution algorithm (1982)
 * Note that it only handles operators and functions
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_TreeNode
extends Object
implements Serializable {
	/* The TreeNode contains essentially a SOM_TypeDeclaration that is info
		 of what the resulting type of the expression is going to be
		 Correspondingly it also stores the argument of the funtion call
		 which may themselves be TreeNodes in a "children" TreeNodeList 
		 */
	
	/** lookUpDecl necessitated by the fact that the overload resolution
		 algorithm would be used also for lookUp on SOM_Names. Hence we needed
		 the corresponding declaration of the SOM_Name to be stored too
		 */
	private SOM_Declaration lookUpDecl;

	/** domainList necessitated by the fact that resolveFinalType needs to 
	 * remember the original "signature" of the formalname. Thus this
	 * domainList is the domain signature for which the children of
	 * this node are created 
	 */
	private SOM_DeclarationList domainList;

	private SOM_TypeDeclaration resultType;
	private SOM_TreeNodeList children = new SOM_TreeNodeList();
	private boolean isAmbiguous = false;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_TreeNode() {
		super();
	}

	/** The second Constructor
	 * @return Not applicable
	 */
	public SOM_TreeNode(SOM_TypeDeclaration typeIn) {
		super();
		resultType = typeIn;
	}

	/** To set the lookUpDecl
	 * @param declIn The SOM_Declaration to be set as the lookUpDecl
	 * @return void
	 */
	public void setLookUpDecl(SOM_Declaration declIn) {
		lookUpDecl = declIn;
	}

	/** To get the lookUpDecl
	 * @return The SOM_Declaration to set as the lookUpDecl
	 */
	public SOM_Declaration getLookUpDecl() {
		return lookUpDecl;
	
	}

	/** To set the domainList
	 * @param declIn The SOM_DeclarationList to be set as the domainList
	 * @return void
	 */
	public void setDomainList(SOM_DeclarationList declIn) {
		domainList = declIn;
	}

	/** To get the domainList
	 * @return The SOM_DeclarationList that is the domainList
	 */
	public SOM_DeclarationList getDomainList() {
		return domainList;
	}

	/** To set the resultType
	 * @param typeIn The SOM_TypeDeclaration to be pointed to
	 * @return void
	 */
	public void setResultType(SOM_TypeDeclaration typeIn) {
		resultType = typeIn;
	}

	/** To get the resultType
	 * @return The SOM_Declaration stored here
	 */
	public SOM_TypeDeclaration getResultType() {
		return resultType;
	}

	/** To add a SOM_TreeNode to the children
	 * @param nodeIn The SOM_TreeNode to be added to the children
	 * @return void
	 */
	public void addToChildren(SOM_TreeNode nodeIn) {
		children.addElement(nodeIn);
	}

	/** To retreive an enumeration of the children
	 * @return A ListEnumeration corresponding to the children
	 */
	public ListEnumeration children() {
		return children.elements();
	}

	/** To set the isAmbiguous flag
	 * @param ambig The boolean value to be set for the ambiguous flag
	 * @return void
	 */
	public void setIsAmbiguous(boolean ambig) {
		isAmbiguous = ambig;
	}

	/** To retreive the value of the isAmbiguous flag
	 * @return The boolean valud of isAmbiguous
	 */
	public boolean getIsAmbiguous() {
		return isAmbiguous;
	}
}

